@@ -46,6 +46,7 @@ typedef struct canon_caps {
   const char *lum_adjustment;
   const char *hue_adjustment;
   const char *sat_adjustment;
+  const char *channel_order; /* (in gutenprint notation) 0123 => KCMY,  1230 => CMYK etc. */
 } canon_cap_t;
 
 static const char standard_sat_adjustment[] =
@@ -164,6 +165,8 @@ static const char* control_cmd_MULTIPASS_MP150[] = {
   NULL
 };
 
+static const char iP4500_channel_order[STP_NCOLORS] = {1,2,3,0}; /* CMYK */
+
 static const canon_cap_t canon_model_capabilities[] =
 {
   /* the first printer is used as default in case something has gone wrong in printers.xml */
@@ -177,6 +180,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   /* ******************************** */
@@ -204,6 +208,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -217,6 +222,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -230,6 +236,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon  BJC 85  *//* heads: BC-20 BC-21 BC-22 */
@@ -242,6 +249,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -255,6 +263,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -268,6 +277,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -281,6 +291,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -294,6 +305,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -307,6 +319,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 8200 *//* heads: BC-50 */
@@ -319,6 +332,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon S500 */
@@ -331,6 +345,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -351,6 +366,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 240 *//* heads: BC-02 BC-05 BC-06 */
@@ -363,6 +379,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 250 *//* heads: BC-02 BC-05 BC-06 */
@@ -375,6 +392,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 1000 *//* heads: BC-02 BC-05 BC-06 */
@@ -387,6 +405,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 2000 *//* heads: BC-20 BC-21 BC-22 BC-29 */
@@ -399,6 +418,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 3000 *//* heads: BC-30 BC-33 BC-34 */
@@ -411,6 +431,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 6100 *//* heads: BC-30/BC-31 BC-32/BC-31 */
@@ -423,6 +444,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 7000 *//* heads: BC-60/BC-61 BC-60/BC-62   ??????? */
@@ -435,6 +457,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC i560 */
@@ -447,6 +470,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 7100 *//* heads: BC-60/BC-61 BC-60/BC-62   ??????? */
@@ -459,6 +483,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC i80 *//* heads: BC-60/BC-61 BC-60/BC-62   ??????? */
@@ -471,6 +496,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 
@@ -491,6 +517,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 5500 *//* heads: BC-20 BC-21 BC-29 */
@@ -503,6 +530,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 6500 *//* heads: BC-30/BC-31 BC-32/BC-31 */
@@ -515,6 +543,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon BJC 8500 *//* heads: BC-80/BC-81 BC-82/BC-81 */
@@ -527,6 +556,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_default_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iP2000 */
@@ -539,6 +569,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iP3000 */
@@ -551,6 +582,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iP4000 */
@@ -563,6 +595,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* PIXMA MP740 (== iP4000 without duplex) */
@@ -576,6 +609,7 @@ static const canon_cap_t canon_model_capabilities[] =
     NULL,
     NULL,
     NULL,
+    NULL
   },
   { /* Canon PIXMA iP5300, MP610 */
     "PIXMA iP5300", 3,          /*model, model_id*/
@@ -587,6 +621,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iP4500 */
@@ -599,7 +634,8 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
-    NULL
+    NULL,
+    iP4500_channel_order
   },
   { /* Canon PIXMA iP4200 */
     "PIXMA iP4200", 3,          /*model, model_id*/
@@ -623,6 +659,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iP6700 */
@@ -635,6 +672,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iX5000 */
@@ -642,11 +680,12 @@ static const canon_cap_t canon_model_capabilities[] =
     842, 17*72,       /* max paper width and height */
     10, 10, 15, 15,    /*border_left, border_right, border_top, border_bottom */
     &canon_PIXMA_iP4000_slotlist,
-    CANON_CAP_STD0|CANON_CAP_DUPLEX|CANON_CAP_r|CANON_CAP_px,0x61,control_cmd_PIXMA_iP4000,  /*features */
+    CANON_CAP_STD0|CANON_CAP_DUPLEX|CANON_CAP_r|CANON_CAP_px|CANON_CAP_I,0x61,control_cmd_PIXMA_iP4000,  /*features */
     &canon_PIXMA_iX5000_modelist,
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA MP520 */
@@ -659,6 +698,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA Pro9500 */
@@ -671,6 +711,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon PIXMA iP8500 */
@@ -683,6 +724,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   { /* Canon MULTIPASS MP150 */
@@ -695,6 +737,7 @@ static const canon_cap_t canon_model_capabilities[] =
     &canon_PIXMA_iP4000_paperlist,
     NULL,
     NULL,
+    NULL,
     NULL
   },
 };
